### Multinomial Logit model estimation

### データファイルの読み込み
Data <- read.csv("ensyu.csv",header=T)
## データ数:Dataの行数を数える
hh <- nrow(Data)

## パラメータの初期値の設定
b0 <- numeric(3)

##### Logit model の対数尤度関数の定義 #####

fr <- function(x) {
  ### パラメータの宣言：
  ## 定数項
  b1 <- x[1]
  
  ## 説明変数
  d1 <- x[2] ## 目的
  #d2 <- x[3] ## 退社時刻
  d3 <- x[3] ## 休前日ダミー
  #d4 <- x[5] ## 通勤距離
  #d5 <- x[4] ## 若者ダミー
  #d6 <- x[7] ## 男性ダミー
  #d7 <- x[4] ## 終電ダミー
  #d8 <- x[4] ## 禁酒ダミー
  
  ## 対数尤度のための変数を宣言
  LL = 0
  
  ### 今回用いる目的地は以下の2つ．
  ## 自宅(home)
  ## 寄道(detour)
  
  ## 効用の計算:説明変数にしたい列を入れる．
  home    <- exp(b1 * matrix(1,nrow =hh,ncol=1))
  detour  <- exp(d1 * (26 - Data$退社時刻) + d3 * Data$休前日ダミー)
  
  ### 選択確率の計算
  ## 分母となる，各々のexp(V)の和をつくる
  deno <- home + detour
  
  ## それぞれ計算する
  Phome     <- home / deno
  Pdetour   <- detour   / deno
  
  ## 選択確率が0になってしまった場合に起こる問題の回避
  Phome     <- (Phome!=0) * Phome + (Phome == 0)
  Pdetour   <- (Pdetour!=0) * Pdetour + (Pdetour == 0)
  
  
  ## 選択結果
  Chome     <- Data$寄り道選択 == 0
  Cdetour   <- Data$寄り道選択 == 1
  
  ## 対数尤度の計算
  LL <- colSums(Chome * log(Phome) + Cdetour * log(Pdetour))
  
}

##### 対数尤度関数frの最大化#####

##パラメータ値の最適化 
res <- optim(b0,fr, method = "Nelder-Mead", hessian = TRUE, control=list(fnscale=-1))

## パラメータ推定値、ヘッセ行列 
b   <- res$par
hhh <- res$hessian

## t値の計算
tval <- b/sqrt(-diag(solve(hhh)))

## 初期尤度
L0 <- fr(b0)
## 最終尤度
LL <- res$value

##### 結果の出力 #####
print(res)
## 初期尤度
print(L0)
## 最終尤度
print(LL)
##ρ^2値
print((L0-LL)/L0)
## 修正済ρ^2値
print((L0-(LL-length(b)))/L0)
##パラメータ推定値 
print(b)
## t値 
print(tval)
